/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.lang.reflect.Constructor;
import java.nio.charset.Charset;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

import com.csvreader.CsvReader;
import com.vaadin.ui.Label;
import com.vaadin.ui.Upload;
import com.vaadin.ui.Upload.FinishedEvent;
import com.vaadin.ui.Window.Notification;

/**
 * Loads data from a CSV file. Upload component helper to process a CSV file
 * with Entity data (Group, Student, Subject, Teacher ...).
 * 
 * 
 * 
 * @author vcarceler
 */
@SuppressWarnings("serial")
public class CSVUploader implements Upload.Receiver, Upload.FinishedListener {

	private CsvReader incsv;
	public int successfulRecordsCounter;
	public int failedRecordsCounter;
	public String MIMEType;
	
	private ByteArrayOutputStream baos;
	private Constructor<? extends Entity> entitieConstructor;
	private String constructorProperties[];
	private String alert;
	
	public CSVUploader(Constructor<? extends Entity> entitieConstructor, String constructorProperties[]) {
		
		this.entitieConstructor = entitieConstructor;
		this.constructorProperties = constructorProperties;
	}

	@Override
	public OutputStream receiveUpload(String filename, String MIMEType) {
		
		Logger.log("CSVUploader(" + filename + ", " + MIMEType + ")");
		
		successfulRecordsCounter = 0;
		failedRecordsCounter = 0;
		baos = new ByteArrayOutputStream();
		OutputStream pos = null;
		this.MIMEType = MIMEType;
		
		if(MIMEType.equals("text/csv")) pos = new CSVImporterOutputStream();
		
		return pos;
	}
	
	/**
	 * Evento de carga finalizada. Lee el csv cargado, genera instancias del objeto importado y lo inserta en la base de datos.
	 * Controla que los datos del csv correspondan con el tipo de objeto que se desea insertar, y muestra advertencias en caso contrario.
	 */
	@Override
	public void uploadFinished(FinishedEvent event) {
		try{
			Logger.log("CSVUploader.uploadFinished()");
			Logger.log("baos.size: " + baos.size());
			Logger.log("File content:\n" + baos.toString());

			incsv = new CsvReader(new ByteArrayInputStream(baos.toByteArray()), ',', Charset.forName("UTF-8"));
			incsv.readHeaders();
			String headers[] = incsv.getHeaders();
			
			if(!(headers.length>=constructorProperties.length)){
				if(MIMEType.equals("text/csv"))	raiseAlert();				
				throw new Exception("Invalid format: not enough columns !!!");
			}
			
			for(int i=0; i<constructorProperties.length; i++) {
				if (!constructorProperties[i].equals(headers[i])){
					if(MIMEType.equals("text/csv"))	raiseAlert();					
					throw new Exception("Invalid format: " + headers[i]);
				}				
			}
			
			while (incsv.readRecord()) {
				String properties[] = new String[constructorProperties.length];
				for(int i=0; i<constructorProperties.length; i++) {
					properties[i] = incsv.get(constructorProperties[i]);
				}
				
				Entity e = entitieConstructor.newInstance((Object [])(properties));
				
				Logger.log("Built: " + e);
				
				if (e.add() != 0) successfulRecordsCounter++; else failedRecordsCounter++;							
			}
			incsv.close();
			
		} catch (Exception e) {
			System.err.println("CSVUploader.uploadFinished(): Error !!!");
			e.printStackTrace();
		}
	}
	
	/**
	 * Cambia el valor de la propiedad alert. Recoge la cadena del mensaje de advertencia y lo asigna a la propiedad alert.
	 * @param alert
	 */
	public void setAlert(String alert){
		this.alert = alert;
	}
	
	/**
	 * Devuelve la propiedad alert
	 * @return alert
	 */
	public String getAlert(){
		return alert;
	}
	
	/**
	 * Crea un mensaje de advertencia de Vaadin con la cadena alert que tenga.
	 */
	public void raiseAlert(){
		Label alertLabel = new Label();		
		alertLabel.setValue(alert);
		alertLabel.setContentMode(Label.CONTENT_XHTML);
		AttendancemanagerApplication.adminWindow.showNotification("CSV INCOMPATIBLE",(String) alertLabel.getValue(), Notification.TYPE_WARNING_MESSAGE);		
	}
	
	/**
	 * Devuelve un Label de Vaadin cuyo contenido es el de la cadena recibida convertido al formato HTML.
	 * @return label
	 */
	public Label getLabelFromString(String message){
		Label label = new Label();
		label.setValue(message);
		label.setContentMode(Label.CONTENT_XHTML);
		return label;
	}
	
	private class CSVImporterOutputStream extends OutputStream {
		
		@Override
		public void write(int b) {
			baos.write(b);
		}		
	}
}
